/**
 * @file   comm_test.c
 * @author Ondrej Jerabek
 */

#include "comm_test.h"
#include "main.h"
#include "pindef.h"
#include "config.h"
#include "extcomm.h"
#include "modules.h"

#include <pck.h>
#include <stdlib.h>
#include <globalrqstat.h>
#include <packet_handle.h>


// --------------------------------------------------------------------------
CommTestSettStruct ctest_settings;
CommTestResultStruct ctest;

unsigned char answer_waiting;
unsigned char data[MAX_PACKET_DATA_SIZE];
volatile unsigned char id;
// --------------------------------------------------------------------------

/**
 * @brief   	Checking if detected answer is correct
 * @param[in]   pck:	Pointer to received packet
 * @return  	Nothing
 */
void ctest_answer_detected(TPacket *pck)
{
	unsigned char i, err = 0;

	if (!answer_waiting)
		return;

	if (pck->Data_Len != ctest_settings.data_len)
		err = 1;
	else
	{
		for (i = 0; i < ctest_settings.data_len; i++)
		{
			if (pck->Buffer[i] != data[i])
				err = 1;
		}
	}

	if (!err)
		ctest.ok++;
	else
		ctest.nok++;

	ctest.completed = 100 * (ctest.nok + ctest.ok) / ctest_settings.cnt;
	answer_waiting = 0;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Checking if response on send answer timeouts
 * @return  	Nothing
 */
void ctest_answer_tout(void)
{
	if (!answer_waiting)
		return;

	ctest.nok++;
	ctest.completed = 100 * (ctest.nok + ctest.ok) / ctest_settings.cnt;
	answer_waiting = 0;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Generate random data pattern of specified length
 * @param		buffer:		Pointer to buffer for saving data
 * @param		len:		Generated data length
 * @return  	Nothing
 */
void ctest_data_generate(unsigned char *buffer, unsigned char len)
{
	while(len)
	{
		*buffer = rand();
		buffer++;
		len--;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   	Send packet for communication test
 * @return  	Nothing
 */
void ctest_send(void)
{
	enum e_packet_source_device device;
	TPacket pck;
	pck.Addr_Src = core.MyAddr;
	pck.Addr_Dest = ctest_settings.addr;
	pck.Rq_St = RQ_ECHO;
	pck.Pck_Type = ROUTING(RTD_BOTH) | (ID_MASK & id++);
	pck.Data_Len = ctest_settings.data_len;
	pck.Buffer = data;
	pck.Protocol = CPT_UNDEFINED;

	device = routing_table_find(ctest_settings.addr);

	comm.answer_id = pck.Pck_Type & ID_MASK;
	comm.source_dev = PSD_UNKNOWN;
	comm.addr_dest = pck.Addr_Dest;

	if (device == PSD_WIRELESS_0)
		module_pck_send(MODULE_1, &pck);
	else if (device == PSD_WIRELESS_1)
		module_pck_send(MODULE_2, &pck);
	else
	{
		module_pck_send(MODULE_1, &pck);
		module_pck_send(MODULE_2, &pck);
	}
	set_answer_timer(ctest_settings.timeout);
	answer_waiting = 1;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Start communication test
 * @param[in]   set:	Pointer to settings of communication test
 * @return  	Nothing
 */
void ctest_start(CommTestSettStruct * set)
{
	ctest_settings.addr = set->addr;
	ctest_settings.cnt = set->cnt;
	ctest_settings.data_len = set->data_len;
	ctest_settings.timeout = set->timeout;

	ctest.completed = 0;
	ctest.in_progress = 1;
	ctest.nok = 0;
	ctest.ok = 0;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Stop active communication test
 * @return  	Nothing
 */
void ctest_stop(void)
{
	ctest.in_progress = 0;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Write communication test results to buffer
 * @param[out]  buff:	Buffer for writing communication test result
 * @return  	Nothing
 */
void ctest_get_result(unsigned char *buff)
{
	memcpy(buff, &ctest, sizeof(ctest));
}
// --------------------------------------------------------------------------

/**
 * @brief   	Handling active communication test
 * @return  	Nothing
 */
void ctest_handle(void)
{
	if (ctest.in_progress)
	{
		if (!answer_waiting && !comm.awaiting_answer)		// Dokoncen prijem odpovedi
		{
			if (ctest_settings.cnt > (ctest.ok + ctest.nok))	// Odeslany vsechny pakety?
			{
				ctest_data_generate(data, ctest_settings.data_len);	// Generovani dalsiho packetu
				ctest_send();
				answer_waiting = 1;
			}
			else
				ctest.in_progress = 0;	// Vsechny pakety odeslany
		}
	}
}
// --------------------------------------------------------------------------
