/**
 * @file   extcomm.c
 * @author Ondrej Jerabek
 */

#include "main.h"
#include "pindef.h"
#include "config.h"
#include "extcomm.h"
#include "cfg.h"
#include "modules.h"
#include "routing.h"
#include "comm_test.h"
#include "rssi.h"

#include <stdio.h>
#include <simplelpcio.h>
#include <system.h>
#include <eeprom.h>
#include <delay.h>
#include <sx_rssi.h>
#include <packet_handle.h>
#include <interrupt.h>
#include <multiuart.h>
#include <multiuart_packet.h>
#include <usb_packet.h>
#include <udp_packet.h>
#include <tcp_packet.h>
#include <tcplink.h>
#include <cdc_vcom.h>
#include <wdt.h>

#define ANSWER_COND(s)	(s == PCK_OK)

// --------------------------------------------------------------------------
/*global variable */
unsigned char outbuff[MAX_PACKET_DATA_SIZE];
CommStruct comm;

// --------------------------------------------------------------------------

/**
 * @brief   Handling packet answer each millisecond
 * @return  Nothing
 * @note	Should be called each millisecond in SysTick interrupt
 */
void answer_ms_tick(void)
{
	if (comm.answer_tmr)
		if (--comm.answer_tmr == 0)
			if (comm.awaiting_answer)
				comm.awaiting_answer = 0, comm.answer_timeout_flag = 1;
}
// --------------------------------------------------------------------------

/**
 * @brief   Set and start answer timer
 * @param	timeout:	Answer timeout in milliseconds
 * @return  Nothing
 */
void set_answer_timer(unsigned long timeout)
{
	di();
	comm.answer_tmr = timeout;
	comm.awaiting_answer = (timeout > 0);
	comm.answer_timeout_flag = 0;
	ei();
}
// --------------------------------------------------------------------------

/**
 * @brief   Stop and clear answer timer
 * @return  Nothing
 */
void clear_answer_timer(void)
{
	di();
	comm.answer_tmr = 0;
	comm.awaiting_answer = 0;
	comm.answer_timeout_flag = 0;
	ei();
}
// --------------------------------------------------------------------------

/**
 * @brief   Handle answer waiting timeout
 * @return  Nothing
 */
void answer_timeout_handle(void)
{
	if (comm.answer_timeout_flag)
	{
		comm.answer_timeout_flag = 0;
		if (ctest.in_progress)
		{
			ctest_answer_tout();
			return;
		}
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Handle packets incoming through semtech module 1
 * @return  Nothing
 */
enum e_pck_handle_status sx1_pck_handle(void)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat;

	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = wireless_pck_detect_1(&packet);

	if (pck_stat == PCK_DETECTED)
	{
		rssi_table_add(packet.Addr_Src, MODULE_1, module_get_rssi(MODULE_1));
		led.mod1_blink = true;

		pck_stat = packet_handle(PSD_WIRELESS_0, packet, &out_packet);
		if (pck_stat == PCK_OK)
		{
			module_pck_send(MODULE_1, &out_packet);
		}
	}

	else if (pck_stat == PCK_NO_REC)
		st.module1_rssi_back = module_get_rssi_background(MODULE_1);

	return pck_stat;
}
// --------------------------------------------------------------------------

/**
 * @brief   Handle packets incoming through semtech module 2
 * @return  Nothing
 */
enum e_pck_handle_status sx2_pck_handle(void)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat;

	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = wireless_pck_detect_2(&packet);

	if (pck_stat == PCK_DETECTED)
	{
		rssi_table_add(packet.Addr_Src, MODULE_2, module_get_rssi(MODULE_2));
		led.mod2_blink = true;

		pck_stat = packet_handle(PSD_WIRELESS_1, packet, &out_packet);
		if (pck_stat == PCK_OK)
		{
			module_pck_send(MODULE_2, &out_packet);
		}
	}

	else if (pck_stat == PCK_NO_REC)
		st.module2_rssi_back = module_get_rssi_background(MODULE_2);

	return pck_stat;
}
// --------------------------------------------------------------------------

/**
 * @brief   Handling packets from system uart
 * @return  Result of packet processing
 */
enum e_pck_handle_status uart_sys_pck_handle(void)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat;

	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = uart_pck_detect(UART_SYS, &packet);

	if (pck_stat == PCK_DETECTED)
	{
		led.sys += 5;
		routing_table_update(packet.Addr_Src, PSD_UART);
		pck_stat = packet_handle(PSD_UART, packet, &out_packet);

		if (pck_stat == PCK_OK)
		{
			uart_pck_send(UART_SYS, &out_packet);
		}
	}
	return pck_stat;
}
// --------------------------------------------------------------------------

/**
 * @brief   Module 1 packets and communication handling
 * @return  Nothing
 */
void module1_handle(void)
{
	if (modules.slot1_type == MODULE_SX)
		sx1_pck_handle();

	else if (modules.slot1_type == MODULE_DW)		// TODO: Write handler for DW
		return;

	else if (modules.slot1_type == MODULE_NTR)
		nanotron_task(nanotron_get_ptr(0));
}
// --------------------------------------------------------------------------

/**
 * @brief   Module 2 packets and communication handling
 * @return  Nothing
 */
void module2_handle(void)
{
	if (modules.slot2_type == MODULE_SX)
		sx2_pck_handle();

	else if (modules.slot2_type == MODULE_DW)		// TODO: Write handler for DW
		return;

	else if (modules.slot2_type == MODULE_NTR)
		nanotron_task(nanotron_get_ptr(1));
}
// --------------------------------------------------------------------------

/**
 * @brief   Handling packets from USB
 * @return  Result of packet processing
 */
enum e_pck_handle_status usb_pck_handle(void)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat;

	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = usb_pck_detect(&packet);

	if (pck_stat == PCK_DETECTED)
	{
		routing_table_update(packet.Addr_Src, PSD_USB);
		pck_stat = packet_handle(PSD_USB, packet, &out_packet);

		if (pck_stat == PCK_OK)
		{
			usb_pck_send(&out_packet);
		}
	}
	return pck_stat;
}
// --------------------------------------------------------------------------

/**
 * @brief   Handling packets from TCP
 * @return  Result of packet processing
 */
enum e_pck_handle_status tcp_pck_handle(unsigned char channel)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat = PCK_NO_REC;

	tcp_channel = channel;
	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = tcp_pck_detect(&packet);
	if (pck_stat == PCK_DETECTED)
	{
		led.conn_blink = 1;
		routing_table_update(packet.Addr_Src, PSD_TCP);
		pck_stat = packet_handle(PSD_TCP, packet, &out_packet);

		if (ANSWER_COND(pck_stat))
			tcp_pck_send(&out_packet);
	}

	return pck_stat;
}
// --------------------------------------------------------------------------

/**
 * @brief   Handling packets from UDP
 * @return  Result of packet processing
 */
enum e_pck_handle_status udp_pck_handle(unsigned char channel)
{
	static unsigned char buff[MAX_PACKET_DATA_SIZE];
	static TPacket packet, out_packet;
	enum e_pck_handle_status pck_stat = PCK_NO_REC;

	packet.Buffer = buff;
	out_packet.Buffer = outbuff;

	pck_stat = udp_pck_detect(&packet);

	if (pck_stat == PCK_DETECTED)
	{
		led.conn_blink = 1;
		//routing_table_update(packet->Addr_Src, PSD_UDP);
		pck_stat = packet_handle(PSD_UDP, packet, &out_packet);

		if (ANSWER_COND(pck_stat))
			udp_pck_send(channel, &out_packet);
	}

	return pck_stat;
}

// --------------------------------------------------------------------------

/**
 * @brief   	Sending wakeup burst
 * @param   	src_addr:	Address of source device
 * @param   	dest_addr:	Address of destination device
 * @param   	count:		Number of packet in burst
 * @param   	rq:   		Request send in packets
 * @param   	data_len:  	Length of data send in packet
 * @param[in]   data:  		Pointer to data
 * @return  	@c true if packet successfully send
 */
bool send_wakeup_burst(Addr_t src_addr, Addr_t dest_addr, unsigned short count, unsigned char rq, unsigned char data_len, unsigned char * data)
{
	static TPacket pck;
	static unsigned char buffer[10];
	static unsigned short cnt;

	if (src_addr != 0 && dest_addr != 0)	// Save data for first send
	{
		if (data_len > 10)
			return 0;
		else
			memcpy(buffer, data, data_len);

		cnt = count;

		pck.Addr_Dest = dest_addr;
		pck.Addr_Src = src_addr;
		pck.Buffer = buffer;
		pck.Data_Len = data_len;
		pck.Rq_St = rq;
		pck.Protocol = CPT_UNDEFINED;
	}

	if (cnt)
	{
		if (modules.slot1_type == MODULE_SX) module_pck_send(MODULE_1, &pck), led.mod1_blink = true;
		if (modules.slot2_type == MODULE_SX) module_pck_send(MODULE_2, &pck), led.mod2_blink = true;
		cnt--;
		return 1;
	}
	else
		return 0;

}
// --------------------------------------------------------------------------

/**
 * @brief   	Internal Service device requests handling
 * @param   	source_device:	Device peripheral which received packet (USB, UART, etc.)
 * @param   	receive_status:	Status of received packet
 * @param[in, out]	pck:		Pointer to received packet. Used for answer packet too
 * @return  	Result of packet processing
 */
enum e_pck_handle_status is_dev_handle(enum e_packet_source_device source_device, enum e_pck_handle_status receive_status, TPacket * pck)
{
	STRUCT_HEADER *sh;
	unsigned char module, s_req;
	unsigned long frequency;

	switch (pck->Rq_St & RQ_MASK)
	{
		case RQ_READ_CFG :
			sh = (STRUCT_HEADER*)pck->Buffer;
			module = sh->struct_type >= CFG_MODULE2_SX;
			if(((pck->Data_Len == sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_DEVICE)) || (pck->Data_Len == 0))
			{
				memcpy(pck->Buffer, &cfg, sizeof(EthernetReader_Cfg));
				pck->Data_Len = sizeof(EthernetReader_Cfg);
				pck->Rq_St = STAT_OK;
			}
			else if ((pck->Data_Len == sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_SX || sh->struct_type == CFG_MODULE2_SX))
			{
				memcpy(pck->Buffer, &module_cfg[module].sx, sizeof(EthernetReader_SX_Cfg));
				pck->Data_Len = sizeof(EthernetReader_SX_Cfg);
				pck->Rq_St = STAT_OK;
			}
			else if ((pck->Data_Len == sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_NTR || sh->struct_type == CFG_MODULE2_NTR))
			{
				memcpy(pck->Buffer, &module_cfg[module].ntr, sizeof(EthernetReader_NTR_Cfg));
				pck->Data_Len = sizeof(EthernetReader_NTR_Cfg);
				pck->Rq_St = STAT_OK;
			}
			else if ((pck->Data_Len == sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_DW || sh->struct_type == CFG_MODULE2_DW))
			{
				memcpy(pck->Buffer, &module_cfg[module].dw, sizeof(EthernetReader_DW_Cfg));
				pck->Data_Len = sizeof(EthernetReader_DW_Cfg);
				pck->Rq_St = STAT_OK;
			}
			else
			{
				pck->Data_Len = 0;
				pck->Rq_St = STAT_DATA_ERR;
			}
			break;

		case RQ_WRITE_CFG :
			sh = (STRUCT_HEADER*)pck->Buffer;
			module = sh->struct_type >= CFG_MODULE2_SX;
			if((pck->Data_Len >= sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_DEVICE))
			{
				if (pck->Data_Len < sizeof(EthernetReader_Cfg))
					memcpy(&cfg, pck->Buffer, pck->Data_Len);
				else
					memcpy(&cfg, pck->Buffer, sizeof(EthernetReader_Cfg));

				flags.reinit_modules = true;
				flags.update_cfg = true;
				pck->Data_Len = 0;
				pck->Rq_St = STAT_OK;
			}
			else if((pck->Data_Len >= sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_SX || sh->struct_type == CFG_MODULE2_SX))
			{
				if (pck->Data_Len < sizeof(EthernetReader_SX_Cfg))
					memcpy(&module_cfg[module].sx, pck->Buffer, pck->Data_Len);
				else
					memcpy(&module_cfg[module].sx, pck->Buffer, sizeof(EthernetReader_SX_Cfg));

				flags.update_cfg = true;
				pck->Data_Len = 0;
				pck->Rq_St = STAT_OK;
			}
			else if((pck->Data_Len >= sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_NTR || sh->struct_type == CFG_MODULE2_NTR))
			{
				if (pck->Data_Len < sizeof(EthernetReader_NTR_Cfg))
					memcpy(&module_cfg[module].ntr, pck->Buffer, pck->Data_Len);
				else
					memcpy(&module_cfg[module].ntr, pck->Buffer, sizeof(EthernetReader_NTR_Cfg));

				flags.update_cfg = true;
				pck->Data_Len = 0;
				pck->Rq_St = STAT_OK;
			}
			else if((pck->Data_Len >= sizeof(STRUCT_HEADER)) && (sh->struct_type == CFG_MODULE1_DW || sh->struct_type == CFG_MODULE2_DW))
			{
				if (pck->Data_Len < sizeof(EthernetReader_DW_Cfg))
					memcpy(&module_cfg[module].dw, pck->Buffer, pck->Data_Len);
				else
					memcpy(&module_cfg[module].dw, pck->Buffer, sizeof(EthernetReader_DW_Cfg));

				flags.update_cfg = true;
				pck->Data_Len = 0;
				pck->Rq_St = STAT_OK;
			}
			else
			{
				pck->Data_Len = 0;
				pck->Rq_St = STAT_DATA_ERR;
			}
			break;

		case RQ_LOAD_CFG :
			sh = (STRUCT_HEADER*)pck->Buffer;
			if (pck->Data_Len == sizeof(STRUCT_HEADER))
				load_configuration(sh->struct_type);
			else
				load_configuration(CFG_ALL);

			pck->Data_Len = 0;
			pck->Rq_St = STAT_OK;
			break;

		case RQ_SAVE_CFG :
			sh = (STRUCT_HEADER*)pck->Buffer;
			if (pck->Data_Len == sizeof(STRUCT_HEADER))
				save_configuration(sh->struct_type);
			else
				save_configuration(CFG_ALL);
			pck->Data_Len = 0;
			pck->Rq_St = STAT_OK;
			break;

		case RQ_GET_STATE :
			sh = (STRUCT_HEADER*)pck->Buffer;
			if((pck->Data_Len == 0) || (pck->Data_Len == sizeof(STRUCT_HEADER) && sh->struct_type == ST_DEVICE))
			{
				memcpy(pck->Buffer, &st, sizeof(EthernetReader_State));
				pck->Data_Len = sizeof(EthernetReader_State);
				pck->Rq_St = STAT_OK;
			}
			break;

		case RQ_IS_WIRELESS_BURST :
			if (pck->Data_Len >= 10)		// New wakeup request
			{
				if (pck->Data_Len - 10 <= MAX_WSPACKET_DATA_SIZE)
				{
					flags.wakeup_burst = send_wakeup_burst(pck->Addr_Src, *(Addr_t *) pck->Buffer, *(unsigned short *) &pck->Buffer[4], pck->Buffer[6], pck->Data_Len - 7, &pck->Buffer[7]);
					pck->Data_Len = 0;
					pck->Rq_St = STAT_OK;
				}
				else
				{
					pck->Data_Len = 0;
					pck->Rq_St = STAT_PAR_OUTOFRAN;
				}
			}
			else if (pck->Data_Len >= 7)		// Old wakeup request
			{
				flags.wakeup_burst = send_wakeup_burst(pck->Addr_Src, (Addr_t)(*(unsigned short *)pck->Buffer), *(unsigned short *) &pck->Buffer[2], pck->Buffer[4], pck->Data_Len - 5, &pck->Buffer[5]);
				pck->Data_Len = 0;
				pck->Rq_St = STAT_OK;
			}
			else
			{
				pck->Data_Len = 0;
				pck->Rq_St = STAT_DATA_ERR;
			}
			break;

		case RQ_SERVICE_DEV :
			s_req = pck->Buffer[0];
			switch (s_req)
			{
				case RQ_SERVICE_SET_FREQ :
					frequency = *((unsigned long *)(&pck->Buffer[1]));
					if (frequency < 870000000 && frequency > 860000000)
					{
						module_cfg[0].sx.rf_frequency_1 = frequency;
						module_cfg[0].sx.listen_on_chn2 = false;
						module_cfg[1].sx.rf_frequency_1 = frequency;
						module_cfg[1].sx.listen_on_chn2 = false;
						flags.update_cfg = true;
						pck->Data_Len = 0;
						pck->Rq_St = STAT_OK;
					}
					else
					{
						pck->Data_Len = 0;
						pck->Rq_St = STAT_DATA_ERR;
					}
					break;

				case RQ_SERVICE_SHUTDOWN :
					flags.power_off = true;
					pck->Data_Len = 0;
					pck->Rq_St = STAT_OK;
					break;

				case RQ_SERVICE_CTEST_START :		// Spusti test komunikace
					if (pck->Data_Len == sizeof(CommTestSettStruct) + 1)		// CommTestSettStruct + Service req
					{
						ctest_start((CommTestSettStruct *) &pck->Buffer[1]);
						pck->Data_Len = 0;
						pck->Rq_St = STAT_OK;
					}
					else
					{
						pck->Data_Len = 0;
						pck->Rq_St = STAT_DATA_ERR;
					}
					break;

				case RQ_SERVICE_CTEST_STOP :		// Zastavi test komunikace
					ctest_stop();
					break;

				case RQ_SERVICE_CTEST_GET_RESULT :	// Vrati vysledky posledniho testu
					ctest_get_result(pck->Buffer);
					pck->Data_Len = sizeof(CommTestResultStruct);
					pck->Rq_St = STAT_OK;
					break;

				case RQ_SERVICE_GET_RSSI_TABLE :
					pck->Data_Len = rssi_table_get(pck->Buffer, pck->Buffer[1], pck->Buffer[2]);
					pck->Rq_St = STAT_OK;
					break;

				default:
					break;
			}
			break;

		default:
			pck->Data_Len = 0;
			pck->Rq_St = STAT_DEVRQ_ERR;
			break;
	}

	return PCK_OK;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Device request packet handling
 * @param   	source_device:	Device peripheral which received packet (USB, UART, etc.)
 * @param   	receive_status:	Status of received packet
 * @param[in, out]	pck:		Pointer to received packet. Used for answer packet too
 * @return  	Result of packet processing
 * @note		Called from library after packet reception and when destination address equal with device address
 */
enum e_pck_handle_status dev_handle(enum e_packet_source_device source_device, enum e_pck_handle_status receive_status, TPacket * pck)
{
	switch (pck->Rq_St & DEVID_MASK)
	{
		case DEV_IS_DEVICE :
			return is_dev_handle(source_device, receive_status, pck);

		default:
			pck->Data_Len = 0;
			pck->Rq_St = STAT_DEVID_ERR;
			return PCK_OK;
	}

	return PCK_OK;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Broadcast packet handling
 * @param   	source_device:	Device peripheral which received packet (USB, UART, etc.)
 * @param   	receive_status:	Status of received packet
 * @param   	pck_in:			Incoming packet
 * @param[out]	pck_out:		Pointer to Outcoming packet buffer
 * @return  	Result of packet processing
 * @note		Called from library after broadcast packet reception
 */
enum e_pck_handle_status dev_broadcast_handle(enum e_packet_source_device source_device, enum e_pck_handle_status receive_status, TPacket pck_in, TPacket * pck_out)
{
	return PCK_BROADCAST;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Answer packet handling
 * @param   	source_device:	Device peripheral which received packet (USB, UART, etc.)
 * @param   	receive_status:	Status of received packet
 * @param[in]	pck:	Pointer to incoming packet
 * @return  	Result of packet processing
 * @note		Called only if device waiting for response to packet
 */
enum e_pck_handle_status answer_handle(enum e_packet_source_device source_device, enum e_pck_handle_status receive_status, TPacket *pck)
{
	enum e_pck_handle_status status = PCK_NO_ANSWER;

	if ((source_device == PSD_WIRELESS_0 || source_device == PSD_WIRELESS_1 || source_device == PSD_NANOTRON_1 || source_device == PSD_NANOTRON_2) && comm.awaiting_answer)
	{
		if (pck->Addr_Src == comm.addr_dest && (pck->Pck_Type & ID_MASK) == comm.answer_id)
		{
			clear_answer_timer();
			routing_table_update(pck->Addr_Src, source_device);

			if (ctest.in_progress)		// Pokud je aktivni test komunikace
			{
				ctest_answer_detected(pck);
				return PCK_NOT_ROUTED;
			}

			switch(comm.source_dev)
			{
				case PSD_TCP:
					tcp_pck_send(pck);
					status = PCK_ROUTED;
					break;

				case PSD_UDP:
					//udp_pck_send();
					status = PCK_ROUTED;
					break;

				case PSD_UART:
					uart_pck_send(UART_SYS, pck);
					status = PCK_ROUTED;
					break;

				case PSD_USB:
					usb_pck_send(pck);
					status = PCK_ROUTED;
					break;

				default:
					break;
			}
		}
	}
	else
	{
		if (source_device == PSD_TCP)
		{
			pck->Rq_St = STAT_GEN_ERR;
			pck->Data_Len = 0;
			status = PCK_OK;
		}
	}
	return status;
}
// --------------------------------------------------------------------------

/**
 * @brief   	Routing packet handling
 * @param   	source_device:	Device peripheral which received packet (USB, UART, etc.)
 * @param   	pck_in:			Incoming packet
 * @param[out]	pck_out:		Pointer to Outcoming packet buffer
 * @return  	Result of packet processing
 * @note		Called from library after packet reception and when destination address not equal with device address
 */
enum e_pck_handle_status packet_route(enum e_packet_source_device source_device, TPacket packet_in, TPacket * packet_out)
{
	enum e_pck_handle_status status = PCK_NOT_ROUTED;

	// Is SX frequency different?
	bool freq_not_same = 	(module_cfg[0].sx.listen_on_chn2 ? module_cfg[0].sx.rf_frequency_2 : module_cfg[0].sx.rf_frequency_1) !=
							(module_cfg[1].sx.listen_on_chn2 ? module_cfg[1].sx.rf_frequency_2 : module_cfg[1].sx.rf_frequency_1);

	if (comm.awaiting_answer)
		return answer_handle(source_device, PCK_DETECTED, &packet_in);

	if (ctest.in_progress)
		return PCK_NOT_ROUTED;

	comm.answer_id = packet_in.Pck_Type & ID_MASK;
	comm.source_dev = source_device;
	comm.addr_dest = packet_in.Addr_Dest;

	switch(source_device)
	{
		case PSD_TCP:
		case PSD_UDP:
		case PSD_UART:
		case PSD_USB:
			module_pck_send(MODULE_1, &packet_in);
			if (freq_not_same || modules.slot1_type != modules.slot2_type)	// Prevent transmitting on same frequency
				module_pck_send(MODULE_2, &packet_in);

			set_answer_timer(ANSWER_TIMEOUT);
			status = PCK_ROUTED;
			break;

		case PSD_WIRELESS_0:
			if (cfg.route_from_wireless1)
			{
				if (USB_CDC_has_connected()) usb_pck_send(&packet_in);
				if (tcplink_tcp_connected()) tcp_pck_send(&packet_in);
				uart_pck_send(UART_SYS, &packet_in);
				status = PCK_ROUTED;
			}
			else
				status = PCK_NOT_ROUTED;
			break;

		case PSD_WIRELESS_1:
			if (cfg.route_from_wireless2)
			{
				if (USB_CDC_has_connected()) usb_pck_send(&packet_in);
				if (tcplink_tcp_connected()) tcp_pck_send(&packet_in);
				uart_pck_send(UART_SYS, &packet_in);
				status = PCK_ROUTED;
			}
			else
				status = PCK_NOT_ROUTED;
			break;

		default:
			break;
	}
	return status;
}
// --------------------------------------------------------------------------

/**
 * @brief   Sending beacon broadcast packets
 * @return  Nothing
 */
void beacon_broadcast(void)
{
	static TPacket pck;
	pck.Addr_Src = core.MyAddr;
	pck.Addr_Dest = BROADCAST;
	pck.Pck_Type = ROUTING(RTD_BOTH) | (pck.Pck_Type + 1);
	pck.Rq_St = STAT_IS_TAG_BROADCAST;
	pck.Data_Len = 0;
	pck.Buffer = NULL;
	pck.Protocol = CPT_UNDEFINED;

	module_pck_send(MODULE_1, &pck);
}
// --------------------------------------------------------------------------
