/**
 * @brief   Extended communication functions
 * @file    extcomm.h
 * @author  Ondrej Jerabek
 *
 */

#ifndef EXTCOMM_H
#define EXTCOMM_H

#include <pck.h>

/**
 * @brief 	Parameters for answer detection
 */
typedef
typedef struct {
	volatile unsigned int addr_dest;			/**< Destination device address		*/
	volatile unsigned char source_dev;			/**< Source device address  	   	*/
	volatile unsigned char answer_id;			/**< Packet ID			    	   	*/
	volatile unsigned char sent_request;		/**< Packet request 		       	*/
	volatile unsigned char send_tries;			/**< Number of tries send packet	*/
	volatile unsigned int answer_tmr;			/**< Awaiting answer timer			*/
	volatile bool awaiting_answer;				/**< Awaiting answer flag  			*/
	volatile bool answer_timeout_flag;			/**< Answer timeout flag        	*/
} CommStruct;

// --------------------------------------------------------------------------
extern unsigned char outbuff[MAX_PACKET_DATA_SIZE];
extern CommStruct comm;

// --------------------------------------------------------------------------
/**
 * @brief   Handling packets from system uart
 * @return  Result of packet processing
 */
extern enum e_pck_handle_status uart_sys_pck_handle(void);

/**
 * @brief   Module 1 packets and communication handling
 * @return  Nothing
 */
extern void module1_handle(void);

/**
 * @brief   Module 2 packets and communication handling
 * @return  Nothing
 */
extern void module2_handle(void);

/**
 * @brief   Handling packets from USB
 * @return  Result of packet processing
 */
extern enum e_pck_handle_status usb_pck_handle(void);

/**
 * @brief   Handling packets from TCP
 * @return  Result of packet processing
 */
extern enum e_pck_handle_status tcp_pck_handle(unsigned char channel);

/**
 * @brief   Handling packets from UDP
 * @return  Result of packet processing
 */
extern enum e_pck_handle_status udp_pck_handle(unsigned char channel);

/**
 * @brief   	Sending wakeup burst
 * @param   	src_addr:	Address of source device
 * @param   	dest_addr:	Address of destination device
 * @param   	count:		Number of packet in burst
 * @param   	rq:   		Request send in packets
 * @param   	data_len:  	Length of data send in packet
 * @param[in]   data:  		Pointer to data
 * @return  	@c true if packet successfully send
 */
extern bool send_wakeup_burst(Addr_t src_addr, Addr_t dest_addr, unsigned short count, unsigned char rq, unsigned char data_len, unsigned char * data);

/**
 * @brief   Handling packet answer each millisecond
 * @return  Nothing
 * @note	Should be called each millisecond in SysTick interrupt
 */
extern void answer_ms_tick(void);

/**
 * @brief   Handle answer waiting timeout
 * @return  Nothing
 */
extern void answer_timeout_handle(void);

/**
 * @brief   Set and start answer timer
 * @param	timeout:	Answer timeout in milliseconds
 * @return  Nothing
 */
extern void set_answer_timer(unsigned long timeout);

/**
 * @brief   Stop and clear answer timer
 * @return  Nothing
 */
extern void clear_answer_timer(void);

/**
 * @brief   Sending beacon broadcast packets
 * @return  Nothing
 */
extern void beacon_broadcast(void);

// --------------------------------------------------------------------------
#endif // EXTCOMM_H
