/**
 * @file   main.c
 * @author Ondrej Jerabek
 */

// C libraries
#include <stdio.h>

// project libraries
#include "ARM_Ethernet_Reader.h"
#include "config.h"
#include "main.h"
#include "power_ctrl.h"
#include "modules.h"
#include "cfg.h"
#include "spi_wrapper.h"
#include "extcomm.h"
#include "comm_test.h"
#include "rssi.h"

// LPC libraries
#include "chip.h"
#include "eeprom.h"
#include "gpio_17xx_40xx.h"
#include "enet_17xx_40xx.h"

// global libraries
#include <coremem.h>
#include <system.h>
#include <run_bootloader.h>
#include <device_types.h>
#include <interrupt.h>
#include <cr_section_macros.h>
#include <dallas.h>
#include <watchdog.h>
#include <cdc_main.h>
#include <cdc_vcom.h>
#include <multisx1211.h>
#include <multisx1211_packet.h>
#include <sx_rssi.h>
#include <multiuart.h>
#include <multiuart_packet.h>
#include <networking.h>
#include <udp_packet.h>
#include <phy/generic_phy.h>
#include <phy/mdio_wrapper.h>
#include <ds18b20.h>
#include <systemext.h>
#include <m25p80.h>
#include <adc.h>

// --------------------------------------------------------------------------
/* global variables */
TCoreRAM core;
EthernetReader_State st;
EthernetReader_Cfg cfg;
ModuleCfg module_cfg[2];
Flag_struct flags;
Timer_struct tmr;
Leds_struct led;

static ADC_CLOCK_SETUP_T ADCSetup;
unsigned char info[INFO_SIZE];
volatile uint16_t mdio_result = 0, pruchody = 0;

// --------------------------------------------------------------------------
/**
 * @brief   Handling millisecond system timer interrupt.
 * @return  Nothing
 * @note	Called by SysTick interrupt
 */
void SysTick_Handler(void)
{
	tmr.timer32++;

	if (flags.usb_forced)
		USB_ms_tick();

	handle_led_blink(led.mod1, led.mod1_blink, (modules.slot1_type > MODULE_NONE), LEDY1);
	handle_led_blink(led.mod2, led.mod2_blink, (modules.slot2_type > MODULE_NONE), LEDY2);
	handle_led_blink(led.conn, led.conn_blink, tcplink_server_connected(), LEDG);
	handle_led(led.err, LEDR);
	handle_led(led.pwr, LED_STAT);
	handle_led(led.sys, LED_SYS);
	handle_led(tmr.buzz, BUZZ);

	if (!BUTTON)
	{
		if (tmr.button < 0xFFFF)
			tmr.button++;
		if (tmr.button == BTN_POWER_OFF_TIME)
			flags.power_off = true;
	}
	else
	{
		if (tmr.button > 50)
		{
			tmr.press_time = tmr.button;
		}
		tmr.button = 0;
	}

	handle_timer_flags(tmr.sec, 1000, flags.sec);
	handle_timer_flags(tmr.ms100, 100, flags.ms100);
	handle_timer(tmr.timesync_broadcast);

	wireless_ms_tick_1();
	wireless_ms_tick_2();
	answer_ms_tick();
	ntrx_ms_tick();

	if (tmr.bootloader)
	{
		tmr.bootloader--;
		if (!tmr.bootloader)
			run_bootloader();
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Handling interrupts from modules.
 * @return  Nothing
 */
void GPIO_IRQHandler(void)
{
	switch(modules.slot1_type)
	{
		case MODULE_SX:
			if (LPC_GPIOINT->IO2.STATR & (1 << IRQ01_PIN)) 	// SX IRQ_0 rising edge
			{
				LPC_GPIOINT->IO2.CLR = (1 << IRQ01_PIN);
				sx_rssi_irq0_handle(0, NSS_DATA1 && NSS_CONFIG1 && SX_EE_CS1_PIN && NSS_DATA2 && NSS_CONFIG2 && SX_EE_CS2_PIN);
			}

			if (LPC_GPIOINT->IO2.STATR & (1 << IRQ11_PIN)) 	// SX IRQ_1 rising edge
			{
				LPC_GPIOINT->IO2.CLR = (1 << IRQ11_PIN);
				sx_rssi_irq1_handle(0);
				flags.sx1_irq_detected = true;
			}
			break;
		case MODULE_DW:			// TODO: Write IRQ handler for this module
			LPC_GPIOINT->IO2.CLR = (1 << IRQ01_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ11_PIN);
			break;
		case MODULE_NTR:		// Not specific usage, clear the flags
			LPC_GPIOINT->IO2.CLR = (1 << IRQ01_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ11_PIN);
			break;
		default:				// No module, should not happened, clear flags
			LPC_GPIOINT->IO2.CLR = (1 << IRQ01_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ11_PIN);
			break;
	}
	switch(modules.slot2_type)
	{
		case MODULE_SX:
			if (LPC_GPIOINT->IO2.STATR & (1 << IRQ02_PIN)) 	// SX IRQ_0 rising edge
			{
				LPC_GPIOINT->IO2.CLR = (1 << IRQ02_PIN);
				sx_rssi_irq0_handle(1, NSS_DATA1 && NSS_CONFIG1 && SX_EE_CS1_PIN && NSS_DATA2 && NSS_CONFIG2 && SX_EE_CS2_PIN);
			}

			if (LPC_GPIOINT->IO2.STATR & (1 << IRQ12_PIN)) 	// SX IRQ_1 rising edge
			{
				LPC_GPIOINT->IO2.CLR = (1 << IRQ12_PIN);
				sx_rssi_irq1_handle(1);
				flags.sx2_irq_detected = true;
			}
			break;
		case MODULE_DW:			// TODO: Write IRQ handler for this module
			LPC_GPIOINT->IO2.CLR = (1 << IRQ02_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ12_PIN);
			break;
		case MODULE_NTR:		// Not specific usage, clear the flags
			LPC_GPIOINT->IO2.CLR = (1 << IRQ02_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ12_PIN);
			break;
		default:				// No module, should not happened, clear flags
			LPC_GPIOINT->IO2.CLR = (1 << IRQ02_PIN);
			LPC_GPIOINT->IO2.CLR = (1 << IRQ12_PIN);
			break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Reading 32bit system timer.
 * @return  Value of system timer in [ms]
 */
unsigned long read_system_timer(void)
{
	unsigned int value;
	di();
	value = tmr.timer32;
	ei();
	return value;
}
// --------------------------------------------------------------------------

/**
 * @brief   Initializing internal PLLs and core clock
 * @return  Nothing
 * @note    Overrides core clock initialization in library
 */
void SystemCoreClock_init(void)
{
	// Enable the crystal
	if (!Chip_Clock_IsCrystalEnabled())
		Chip_Clock_EnableCrystal();

	while(!Chip_Clock_IsCrystalEnabled()) {}

	// Clock the CPU from SYSCLK, in case if it is clocked by PLL0
	Chip_Clock_SetCPUClockSource(SYSCTL_CCLKSRC_SYSCLK);

	// Disable the PLL if it is enabled
	if (Chip_Clock_IsMainPLLEnabled()) {
		Chip_Clock_DisablePLL(SYSCTL_MAIN_PLL, SYSCTL_PLL_ENABLE);
	}

	// It is safe to switch the PLL Source to Crystal Oscillator
	Chip_Clock_SetMainPLLSource(SYSCTL_PLLCLKSRC_MAINOSC);

	// Setup System clock PLL (19.2 MHz * 5) = 96 MHz
	Chip_Clock_SetupPLL(SYSCTL_MAIN_PLL, 4, 0);

	Chip_Clock_EnablePLL(SYSCTL_MAIN_PLL, SYSCTL_PLL_ENABLE);
	Chip_Clock_SetCPUClockDiv(1);

	while (!Chip_Clock_IsMainPLLLocked()) {} // Wait for the PLL to Lock
	Chip_Clock_SetCPUClockSource(SYSCTL_CCLKSRC_MAINPLL);

	// Peripheral clocking will be derived from PLL0 with a divider of 1
	Chip_Clock_SetPCLKDiv(1);

	SystemCoreClockUpdate();
}
// --------------------------------------------------------------------------

/**
 * @brief   Initializing 32 MHz clock output for nanotron module.
 * @param   enabled:	Enable clock output
 * @return  Nothing
 */
void SystemClkOut_init(bool enabled)
{
	if (enabled)
		LPC_SYSCTL->CLKOUTCFG = (1 << 8) | (0x2 << 4) | (0x00);	// CLKOUT enabled, divided by 3, CPU clk
	else
		LPC_SYSCTL->CLKOUTCFG = 0;	// CLKOUT disabled
}
// --------------------------------------------------------------------------

/**
 * @brief   Initializing clock for USB peripheral
 * @return  Nothing
 * @note    Overrides clock initialization in USB library
 */
void Chip_USB_Init(void)
{
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_USB);

	while ((Chip_Clock_GetPLLStatus(SYSCTL_MAIN_PLL) & SYSCTL_PLLSTS_LOCKED) == 0) {}

	// Select MAIN PLL as clock source for USB block and divider as 2
	LPC_SYSCTL->USBCLKSEL = (SYSCTL_USBCLKSRC_MAINPLL << 8) | 0x02;

	// Enable AHB clock to the USB block and USB RAM.
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_USB);
}
// --------------------------------------------------------------------------

/**
 * @brief   Reinitialization SX modules if someone unexpectedly restart
 * @return  Nothing
 */
void reinit_SX_modules(void)
{
	if (modules.slot1_type == MODULE_SX && IRQ_11)
	{
		if (!flags.sx1_irq_detected)
		{
			tmr.buzz = 10;
			sx_init_1();
			flags.update_cfg = true;
		}
		else
			flags.sx1_irq_detected = false;
	}

	if (modules.slot2_type == MODULE_SX && IRQ_12)
	{
		if (!flags.sx2_irq_detected)
		{
			tmr.buzz = 10;
			sx_init_2();
			flags.update_cfg = true;
		}
		else
			flags.sx2_irq_detected = false;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Initializing ADC converter for measuring voltages and currents
 * @return  Nothing
 * @note	After initialization start converting in Burst mode
 */
void ADC_init(void)
{
	Chip_ADC_Init(LPC_ADC, &ADCSetup);
	Chip_ADC_EnableChannel(LPC_ADC, V_BATT_CHNL, ENABLE);
	Chip_ADC_EnableChannel(LPC_ADC, V_SUPPLY_CHNL, ENABLE);
	Chip_ADC_EnableChannel(LPC_ADC, V_USB_CHNL, ENABLE);
	Chip_ADC_EnableChannel(LPC_ADC, I_MEAS_CHNL, ENABLE);
	Chip_ADC_SetSampleRate(LPC_ADC, &ADCSetup, 100000);
	Chip_ADC_SetBurstCmd(LPC_ADC, ENABLE);
}
// --------------------------------------------------------------------------

/**
 * @brief   Main function of firmware
 * @return  Ideally Nothing. Never ending function
 */
int main(void) {
	unsigned short i;

	pindef_init();			// Init pins
	POWER(1);				// Keep device powered if it runs from battery

	Chip_WWDT_Init(LPC_WWDT);
	Chip_WWDT_SetTimeOut(LPC_WWDT, 1 * WDT_OSC);		// Watch dog = 1 sec
	Chip_WWDT_SetOption(LPC_WWDT, WWDT_WDMOD_WDRESET);
#ifndef DEBUG
	Chip_WWDT_Start(LPC_WWDT);
#endif

	LPC_SYSCTL->RSID = 0x3F;		// Clear reset type

	for (i = 0; i <= EEPROM_IRQn; i++)		// Set priority to interrupts
		NVIC_SetPriority(i, 1);
	NVIC_SetPriority(GPIO_IRQn, 0);

	set_chrg_mode(CHRG_OFF);		// Disable battery charging

	SystemCoreClock_init();

	eeprom_init();					// Initialize components
	extflash_init();
	system_init();
	load_configuration(CFG_ALL);	// Loading all configuration

	if (core.MyAddr == 0)			// Set device address to 1 if not configured
		core.MyAddr = 1;

	ADC_init();
	SPI_Master_Init(MODULES_SPI, SPI_1_FREQ);
	UART_Init(UART_SYS, UART_SYS_BAUD_RATE);

	flags.usb_forced = false;
	if (cfg.usb_enabled)			// Check if USB enabled and USB cable connected
	{
		while (Chip_ADC_ReadValue(LPC_ADC, V_USB_CHNL, &i) != SUCCESS);
		st.usb_voltage = i * DIVIDER(DIV_USB_R1, DIV_USB_R2);
		if (st.usb_voltage > 4000)
			flags.usb_forced = true;			// Voltage on USB is higher than 4V -> cable connected, enable USB
	}

	networking_init(0);

	if (flags.usb_forced)			// Initialize USB, if enabled
	{
		mdio_write(PHY_REG_BMCR, 1 << 11);	// PHY power down
		Chip_ENET_DeInit(LPC_ETHERNET);		// Disable Ethernet peripheral
		USB_init();					// TODO: USB not working simultaneously with Ethernet. Used only if enabled in configuration and connected
	}

	init_dallas();					// Initialize dallas for temperature measuring
	srand(core.MyAddr);

	tmr.sec = 1000;
	tmr.button = 0xffff;
    SysTick_Config(MS_TICK);

    led.sys = 1000;					// Turn on all LEDs for 1 second
	tmr.buzz = 100;
	led.mod2 = 1000;
	led.mod1 = 1000;
	led.err = 1000;
	led.conn = 1000;
	led.pwr = 1000;
	while(led.pwr > 0) WDT_RESET();
	modules_init();
	tmr.press_time = 0;
	set_chrg_mode(CHRG_500MA);

    while(1) {
    	WDT_RESET();
    	if (flags.usb_forced)		// TODO: USB not working simultaneously with Ethernet. Used only if enabled in configuration and connected
    	{
    		USB_CDC_handle();		// Handling USB communication
        	usb_pck_handle();		// Processing packets from USB
    	}
    	else
    		networking_handle();	// Processing packets from Network
    	uart_sys_pck_handle();		// Processing packets from system UART
    	module1_handle();			// Processing packets from module 1
    	module2_handle();			// Processing packets from module 2
    	answer_timeout_handle();	// Handling answer timeout
    	ctest_handle();				// Handling wireless communication test

    	sx_reset_handle(0);			// Handling SX reset if buffer overflow
    	sx_reset_handle(1);

    	nanotron_beacon();			// Sending Nanotron beacon

    	if (flags.wakeup_burst)		// Sending waking up burst
    	{
    		flags.wakeup_burst = send_wakeup_burst(0, 0, 0, 0, 0, NULL);
    	}

    	if (flags.reinit_modules)	// Reinitialization all modules
    	{
    		flags.reinit_modules = false;
    		modules_init();
    	}

    	if (flags.update_cfg)		// Update configuration if needed
    	{
    		flags.update_cfg = false;
    		update_configuration();
    	}

    	if (flags.ms100)
    	{
    		flags.ms100 = false;
    		pwr_meas_vals();		// Measuring and converting ADC values

    		reinit_SX_modules();
    	}

    	if (flags.sec) {
    		flags.sec = false;
    		st.uptime++;

    		pwr_handle();			// Power & Charging control
    		rssi_table_tick();

    		if (st.uptime % 10 == 0 && !flags.wakeup_burst)	// Measuring temperature each 10 seconds
    		{
    			st.temperature = (TEMP)ds_readtemp(NULL);
    			ds_converttemp(NULL);
    			if (st.temperature != (TEMP)0x8000)
    				st.temperature = st.temperature * 100L / 16;
    		}
    	}

    	if (tmr.press_time)			// Handling button press
    	{
    		if (tmr.press_time > 50)
    		{
    			if (tmr.press_time < BTN_RESET_TIME)
    				NVIC_SystemReset();				// System reset
    		}
    		tmr.press_time = 0;
    	}

    	if (flags.power_off)		// Power off request
    	{
    		flags.power_off = false;
    		pwr_suicide();			// Power off
    	}

    	if (run_bootloader_flag)	// Run bootloader
		{
			run_bootloader_flag = false;
			WDT_RESET();
			BOOTLOADER_MACRO;		// Jump to bootloader after 200 ms
			while(1) {
				if (flags.usb_forced) 	// Still handling USB packets
					usb_pck_handle();
			}
		}
    }
    return 0;
}
