/**
 * @brief   Functions for controlling NanoLOC
 * @file    nanotron.h
 * @author  Ondrej Jerabek
 */

#ifndef NANOTRON_H_
#define NANOTRON_H_

#include <NanoGlobal.h>

// --------------------------------------------------------------------------
/**
 * @brief 	Data request to the next lower layer.
 * @note 	This function calls the next lower layer to request a data transmission.
 */
#define SendMsg PDSap

// --------------------------------------------------------------------------
/**
 * @brief Message structure for data requests.
*/
extern MsgT downMsg;


/**
 * @brief ToA (Time of Arrival) structure
 * @note For ranging functionality implementation
 */
typedef struct {
//	AddrT Addr;
	uint16_t timeslot;
	uint8_t ToaOffsetMeanDataValid;
	uint8_t PhaseOffsetData;
	uint16_t ToaOffsetMeanData;
	/*
	 uint8_t	ToaOffsetMeanAckValid;
	 uint8_t PhaseOffsetAck;
	 uint16_t ToaOffsetMeanAck;
	 uint16_t TxRespTime;
	 */
	uint64_t BaseTimer;
	uint8_t AgcGain;

	uint64_t RTCSecs;
	int16_t RTCMsecs;
} TOA_DATA;

/**
 * @brief   Calibrate all nanoLOC modules
 * @return  Calibration result. True if successful
 */
extern bool nanotron_calibrate_all(void);

/**
 * @brief   Initialize all nanoLOC modules
 * @return  Initialization result. True if successful
 */
extern bool nanotron_init_all(void);

/**
 * @brief   Initialize one nanoLOC module
 * @param	index:	NanoLOC chip index
 * @return  Initialization result. True if successful
 */
extern bool nanotron_init(unsigned int index);

/**
 * @brief   Handling nanoLOC communication
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @return  Nothing
 */
extern void nanotron_task(NanoGlobCTX * ngX);

/**
 * @brief   Calibrate one NanoLOC module
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @return  Calibration result. True if successful
 */
extern bool nanotron_calibrate(NanoGlobCTX * ngX);

/**
 * @brief   Enable NanoLOC receiver
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @return  Nothing
 */
extern void nanotron_rx_enable(NanoGlobCTX * ngX, bool enabled);

/**
 * @brief   Sending RTC broadcast packet through nanoLOC module
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @return  Nothing
 */
extern void nanotron_broadcast_rtc(NanoGlobCTX * ngX);

/**
 * @brief   Enable receiving all packets (Disable address checking)
 * @param	index:		NanoLOC chip index
 * @param	enabled:	Enable/disable receiving all packets
 * @return  Nothing
 */
extern void nanotron_enable_promiscuous(unsigned int index, bool enabled);

/**
 * @brief   Read ToA (Time of Arrival) data structure
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @return  Pointer to TOA data structure
 */
extern TOA_DATA * nanotron_get_toa(NanoGlobCTX * ngX);

/**
 * @brief   Translate nanoLOC index to pointer to NanoLOC work & configuration structure
 * @param	index:	NanoLOC chip index
 * @return  Pointer to NanoLOC work & configuration structure
 */
extern NanoGlobCTX * nanotron_get_ptr(unsigned int index);

/**
 * @brief   Set nanoLOC chip transmitting power
 * @param	ngX:	Pointer to NanoLOC work & configuration structure
 * @param	pwr:	NanoLOC transmit power. Valid value 0 - 63
 * @return  @c true if set succesfully
 */
extern bool nanotron_set_tx_pwr(NanoGlobCTX * ngX, unsigned char pwr);

/**
 * @brief   Sending Beacon through nanotron module
 * @return  Nothing
 */
extern void nanotron_beacon(void);

// --------------------------------------------------------------------------
extern TOA_DATA toa[NTRX_COUNT];

// --------------------------------------------------------------------------
#endif // NANOTRON_H_
