/**
 * @file   rssi.c
 * @author Ondrej Jerabek
 */


#include "rssi.h"
#include "config.h"
#include "main.h"
#include <sx_rssi.h>

// --------------------------------------------------------------------------
rssi_table_t rssi_table[RSSI_TABLE_SIZE];

// --------------------------------------------------------------------------
/**
 * @brief   Handle timers of the items in table
 * @return  Nothing
 * @note	Should be called each second
 */
void rssi_table_tick(void)
{
	unsigned char i;

	for (i = 0; i < RSSI_TABLE_SIZE; i++)
	{
		if (rssi_table[i].timer[0])
		{
			if (--rssi_table[i].timer[0] == 0)
				rssi_table[i].rssi[0] = 0;
		}

		if (rssi_table[i].timer[1])
		{
			if (--rssi_table[i].timer[1] == 0)
				rssi_table[i].rssi[1] = 0;
		}

		if (!rssi_table[i].timer[1] && !rssi_table[i].timer[0])		// Delete record
			rssi_table[i].addr = 0;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Adding item to RSSI table
 * @param	addr:		Device address
 * @param	module:		Module which received packet
 * @param	rssi:		Received packet RSSI
 * @return  Nothing
 */
void rssi_table_add(unsigned int addr, unsigned char module, unsigned char rssi)
{
	unsigned char i, free_pos = 255, min_rssi_val = 255, min_rssi_rec = 255;

	for (i = 0; i < RSSI_TABLE_SIZE; i++)		// Searching records
	{
		if (rssi_table[i].addr != 0)					// minimum RSSI
		{
			if (rssi_table[i].rssi[0] > rssi_table[i].rssi[1])
			{
				if (rssi_table[i].rssi[0] < min_rssi_val)
				{
					min_rssi_val = rssi_table[i].rssi[0];
					min_rssi_rec = i;
				}
			}
			else
			{
				if (rssi_table[i].rssi[1] < min_rssi_val)
				{
					min_rssi_val = rssi_table[i].rssi[1];
					min_rssi_rec = i;
				}
			}
		}

		if (free_pos == 255 && rssi_table[i].addr == 0)	// free position
			free_pos = i;

		if (rssi_table[i].addr == addr)		// same address
			break;
	}

	if (i == RSSI_TABLE_SIZE)	// No record with same address avaible, create new
	{
		if (free_pos == 255)		// No free position, overwrite record with lowest rssi
		{
			if (min_rssi_val < rssi)	// overwrite only if RSSI in table is lower, otherwise discard it
			{
				rssi_table[min_rssi_rec].rssi[0] = 0;
				rssi_table[min_rssi_rec].timer[0] = 0;
				rssi_table[min_rssi_rec].rssi[1] = 0;
				rssi_table[min_rssi_rec].timer[1] = 0;

				rssi_table[min_rssi_rec].addr = addr;
				rssi_table[min_rssi_rec].rssi[module] = rssi;
				rssi_table[min_rssi_rec].timer[module] = RSSI_TIMEOUT;
			}
		}
		else	// Creating new record
		{
			rssi_table[free_pos].rssi[0] = 0;
			rssi_table[free_pos].timer[0] = 0;
			rssi_table[free_pos].rssi[1] = 0;
			rssi_table[free_pos].timer[1] = 0;

			rssi_table[free_pos].addr = addr;
			rssi_table[free_pos].rssi[module] = rssi;
			rssi_table[free_pos].timer[module] = RSSI_TIMEOUT;
		}
	}
	else	// Updating values
	{
		rssi_table[i].rssi[module] = rssi;
		rssi_table[i].timer[module] = RSSI_TIMEOUT;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Put RSSI table to buffer (Prepare it for send)
 * @param	buffer:			Pointer to buffer where RSSI table will be copied
 * @param	index:			Index in RSSI table where the copying starts
 * @param	max_rec_cnt:	Maximum number of items copied to buffer
 * @return  Occupied size in bytes
 */
unsigned char rssi_table_get(unsigned char *buffer, unsigned char index, unsigned char max_rec_cnt)
{
	unsigned char size = 0;

	while (max_rec_cnt > 0)
	{
		if (index > RSSI_TABLE_SIZE)
			break;

		memcpy(buffer, &rssi_table[index], sizeof(rssi_table_t));
		size += sizeof(rssi_table_t);
		buffer += sizeof(rssi_table_t);
		index++;
		max_rec_cnt--;
	}

	return size;
}
