/**
 *	@file   spi_wrapper.c
 * @author Ondrej Jerabek
 */

#include <stdint.h>
#include "config.h"
#include <chip.h>
#include "spi_wrapper.h"
#include "ssp_17xx_40xx.h"

// --------------------------------------------------------------------------
/*global variable */

// --------------------------------------------------------------------------
/**
 * @brief   Initialize SPI in Master mode
 * @param	index:		Index of SPI peripheral
 * @param	freq:		Frequency of SPI clk
 * @return  Nothing
 */
void SPI_Master_Init(unsigned char index, unsigned int freq)
{
	switch (index)
	{
		case 0:
			Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SSP0);
			Chip_SSP_Set_Mode(LPC_SSP0, SSP_MODE_MASTER);
			Chip_SSP_SetFormat(LPC_SSP0, SSP_BITS_8, SSP_FRAMEFORMAT_SPI, SSP_CLOCK_CPHA0_CPOL0);
			Chip_SSP_SetBitRate(LPC_SSP0, freq);
			Chip_SSP_Enable(LPC_SSP0);
			break;

		case 1:
			Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SSP1);
			Chip_SSP_Set_Mode(LPC_SSP1, SSP_MODE_MASTER);
			Chip_SSP_SetFormat(LPC_SSP1, SSP_BITS_8, SSP_FRAMEFORMAT_SPI, SSP_CLOCK_CPHA0_CPOL0);
			Chip_SSP_SetBitRate(LPC_SSP1, freq);
			Chip_SSP_Enable(LPC_SSP1);
			break;

		case 2:
			Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SSP2);
			Chip_SSP_Set_Mode(LPC_SSP2, SSP_MODE_MASTER);
			Chip_SSP_SetFormat(LPC_SSP2, SSP_BITS_8, SSP_FRAMEFORMAT_SPI, SSP_CLOCK_CPHA0_CPOL0);
			Chip_SSP_SetBitRate(LPC_SSP2, freq);
			Chip_SSP_Enable(LPC_SSP2);
			break;

		default :	break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Send and receive data through SPI.
 * @param	pSSP:	Pointer to SPI register
 * @param	data:	Data to send
 * @return  Received data
 */
static unsigned short SPI_read_write_data(LPC_SSP_T *pSSP, unsigned short data)
{
	while (!(Chip_SSP_GetStatus(pSSP, SSP_STAT_TNF)));
	Chip_SSP_SendFrame(pSSP, data);
	while (!(Chip_SSP_GetStatus(pSSP, SSP_STAT_RNE)));
	data = Chip_SSP_ReceiveFrame(pSSP);
	while (Chip_SSP_GetStatus(pSSP, SSP_STAT_BSY))
		;
	return data;
}
// --------------------------------------------------------------------------

/**
 * @brief   Waiting for complete SPI transfer
 * @param	index:		Index of SPI peripheral
 * @return  Nothing
 */
void SPI_Wait_Busy(unsigned char index)
{
	switch (index)
	{
		case 0: while (Chip_SSP_GetStatus(LPC_SSP0, SSP_STAT_BSY));	break;
		case 1: while (Chip_SSP_GetStatus(LPC_SSP1, SSP_STAT_BSY));	break;
		case 2: while (Chip_SSP_GetStatus(LPC_SSP2, SSP_STAT_BSY));	break;
		default : break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Send data through SPI
 * @param	index:		Index of SPI peripheral
 * @param	data:		Data to send
 * @return  Nothing
 */
void SPI_Write(unsigned char index, unsigned short data)
{
	switch (index) {
		case 0: SPI_read_write_data(LPC_SSP0, data);	break;
		case 1: SPI_read_write_data(LPC_SSP1, data);	break;
		case 2: SPI_read_write_data(LPC_SSP2, data);	break;
		default: break;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Send and receive data through SPI
 * @param	index:		Index of SPI peripheral
 * @param	data:		Data to send
 * @return  Received data
 */
unsigned short SPI_Read_Write(unsigned char index, unsigned short data)
{
	switch (index)
	{
		case 0: data = SPI_read_write_data(LPC_SSP0, data);	break;
		case 1: data = SPI_read_write_data(LPC_SSP1, data);	break;
		case 2: data = SPI_read_write_data(LPC_SSP2, data);	break;
		default : break;
	}
	return data;
}
// --------------------------------------------------------------------------
